/**
 *  TestExit for debugging MQ exit issue.
 *
 *  Location of:
 *  Unix/Linux:   /var/mqm/exits64/testexit(SecExit)
 *
 *  Windows DLL:  C:\Program Files\IBM\MQ\Exits64\testexit(SecExit)
 *
 *  MQSC command for Unix/Linux queue manager:
 *  DEFINE CHANNEL ('APP.CH01') CHLTYPE(SVRCONN) +
 *         TRPTYPE(TCP) +
 *         SENDEXIT('/var/mqm/exits64/testexit(SecExit)') +
 *         SENDDATA(' ') +
 *         REPLACE
 *
 *  MQSC command for Windows queue manager:
 *  DEFINE CHANNEL ('APP.CH01') CHLTYPE(SVRCONN) +
 *         TRPTYPE(TCP) +
 *         SENDEXIT('C:\Program Files\IBM\MQ\Exits64\testexit(SecExit)') +
 *         SENDDATA(' ') +
 *         REPLACE
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#if defined (_WIN32)
#include <windows.h>
#include <lm.h>
#endif

#include <cmqc.h>
#include <cmqxc.h>

#if defined (_WIN32)
__declspec (dllexport) void MQENTRY SecExit(PMQCXP  pChannelExitParms,
                                            PMQCD   pChannelDefinition,
                                            PMQLONG pDataLength,
                                            PMQLONG pAgentBufferLength,
                                            PMQBYTE pAgentBuffer,
                                            PMQLONG pExitBufferLength,
                                            PMQPTR  pExitBufferAddr);
#else
extern void MQENTRY MQStart(void) {;}
#endif

void MQENTRY SecExit( PMQCXP  pChannelExitParms,
                      PMQCD   pChannelDefinition,
                      PMQLONG pDataLength,
                      PMQLONG pAgentBufferLength,
                      PMQVOID pAgentBuffer,
                      PMQLONG pExitBufferLength,
                      PMQPTR  pExitBufferAddr)
{
   char       outBuff[512];
   char       tempChl[ MQ_CHANNEL_NAME_LENGTH + 1];
   FILE*      fh;
   struct tm  *newtime;
   time_t     tclock;
   char       *timeBuff;
   PMQCXP     pParms = pChannelExitParms;
   PMQCD      pChDef = pChannelDefinition;

   // Attention newbie user: Make sure you select a valid directory and filename!!!

//   fh = fopen("C:\\Program Files\\IBM\\MQ\\Exits\\testexit.log", "a+");
   fh = fopen("C:\\Temp\\testexit.log", "a+");

   time( &tclock );
   newtime = localtime( &tclock );
   timeBuff = asctime(newtime);
   timeBuff[strlen(timeBuff) - 1] = 0x00;

   fprintf(fh, "%s : Now entering the security exit.\n", timeBuff);

   memcpy(tempChl, pChannelDefinition->ChannelName, MQ_CHANNEL_NAME_LENGTH);
   tempChl[MQ_CHANNEL_NAME_LENGTH] = 0x00;
   fprintf(fh, "%s : Channel name is %s\n", timeBuff, tempChl);

   if (pParms->ExitId == MQXT_CHANNEL_SEC_EXIT)
   {
      switch (pParms->ExitReason)
      {
         case MQXR_INIT:
              pParms->ExitResponse = MQXCC_OK;
              fprintf(fh, "%s : MQXR_INIT - Channel Initialization\n", timeBuff);
              break;
         case MQXR_INIT_SEC:
              pParms->ExitResponse = MQXCC_OK;
              fprintf(fh, "%s : MQXR_INIT_SEC  - Initialize Security\n", timeBuff);
              break;
         case MQXR_SEC_MSG:
              pParms->ExitResponse = MQXCC_OK;
              fprintf(fh, "%s : MQXR_SEC_MSG - Security Message\n", timeBuff);
              break;
         case MQXR_SEC_PARMS:
              pParms->ExitResponse = MQXCC_OK;
              fprintf(fh, "%s : MQXR_SEC_PARMS - Security Parameters\n", timeBuff);
              break;
         case MQXR_TERM:
              pParms->ExitResponse = MQXCC_OK;
              fprintf(fh, "%s : MQXR_TERM - Channel Terminating\n", timeBuff);
              break;
         default:
              pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
              fprintf(fh, "%s : ERROR - Unknown Exit Reason\n", timeBuff);
              break;
      }
   }
   else
   {
      pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
      fprintf(fh, "%s : ERROR - Not invoked by a security exit.\n", timeBuff);
      return;
   }

   fprintf(fh, "%s : Now exiting the security exit.\n\n", timeBuff);
   fclose(fh);

   return;
}
